@@ -15,7 +15,7 @@ has migrations      => sub {
 };
 has url             => sub { Mojo::MySQL5::URL->new('mysql:///test') };
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 sub db {
   my $self = shift;
@@ -28,7 +28,7 @@ sub db {
 
   if (!$c) {
     $db->connect;
-    croak 'connect failed' unless $db->connection->_state eq 'idle';
+    croak 'connect failed' unless $db->connection->state eq 'idle';
     $self->emit(connection => $db);
   }
   return $db;
